@@ -1,11 +1,18 @@
 #!/opt/perl5/bin/perl
 
-use CGI::Switch;
+use CGI ();
+use CGI::Switch ();
 use strict;
 
-my $r = CGI::Switch->new;
+#hack for win32 because ApacheCore.dll does not export basic_http_header,
+#so CGI::Switch doesn't work, but mod_perl-win32 requires 5.004_02+ anyhow,
+#so there's no need for CGI::Switch
+
+my $class = $] >= 5.004 ? "CGI" : "CGI::Switch";
+
+my $r = $class->new;
+
 warn "Running cgi.pl with $CGI::VERSION";
-warn $r;
 
 my($param) = $r->param('PARAM');
 my($httpupload) = $r->param('HTTPUPLOAD');
